SystemOrganization addCategory: #'PersistentUnionFind-Tests'!
SystemOrganization addCategory: #PersistentUnionFind!

Object subclass: #ArrayPart
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind'!

ArrayPart subclass: #Arr
	instanceVariableNames: 'array'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind'!

Arr class
	instanceVariableNames: 'array'!

!Arr commentStamp: 'fbs 12/28/2011 11:12' prior: 0!
An Arr is a wrapper for an array - really, a SequenceableCollection.!

Arr class
	instanceVariableNames: 'array'!

!Arr class methodsFor: 'instance creation' stamp: 'fbs 12/28/2011 11:45'!
wrapping: aSequenceableCollection
	^ self new wrapping: aSequenceableCollection.! !

!Arr methodsFor: 'accessing' stamp: 'fbs 12/28/2011 12:53'!
array
	^ array! !

!Arr methodsFor: 'converting' stamp: 'fbs 12/28/2011 12:38'!
asArray
	^ array copy.! !

!Arr methodsFor: 'printing' stamp: 'fbs 12/28/2011 15:59'!
printOn: aStream
	aStream
		nextPutAll: 'Arr(';
		nextPutAll: array printString;
		nextPutAll: ')'.! !

!Arr methodsFor: 'private' stamp: 'fbs 12/28/2011 19:01'!
wrapping: aSequenceableCollection
	array := aSequenceableCollection copy.! !

!ArrayPart methodsFor: 'converting' stamp: 'fbs 12/29/2011 11:17'!
asArray
	self subclassResponsibility.! !

!ArrayPart methodsFor: 'testing' stamp: 'fbs 12/29/2011 11:16'!
isDiff
	^ false.! !

ArrayPart subclass: #Diff
	instanceVariableNames: 'index value baseArray'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind'!

!Diff commentStamp: 'fbs 12/29/2011 10:44' prior: 0!
A Diff is a difference record. An instance is always relative to some Arr a, and means "like a, only with this element replaced with that value."

In particular, note that baseArray is a MutableCell referencing another PersistentArray.!

!Diff class methodsFor: 'instance creation' stamp: 'fbs 12/28/2011 11:39'!
index: anInteger value: anObject in: aPersistentArray
	^ self new index: anInteger value: anObject in: aPersistentArray.! !

!Diff methodsFor: 'converting' stamp: 'fbs 12/28/2011 12:45'!
asArray
	| arr |
	arr := baseArray asArray.
	arr at: index put: value.
	^ arr.! !

!Diff methodsFor: 'accessing' stamp: 'fbs 12/29/2011 11:08'!
baseArray
	^ baseArray.! !

!Diff methodsFor: 'accessing' stamp: 'fbs 12/28/2011 12:34'!
index
	^ index.! !

!Diff methodsFor: 'private' stamp: 'fbs 12/28/2011 11:40'!
index: anInteger value: anObject in: aPersistentArray
	index := anInteger.
	value := anObject.
	baseArray := aPersistentArray.! !

!Diff methodsFor: 'testing' stamp: 'fbs 12/28/2011 11:27'!
isDiff
	^ true.! !

!Diff methodsFor: 'printing' stamp: 'fbs 12/28/2011 15:59'!
printOn: aStream
	aStream
		nextPutAll: 'Diff(';
		nextPutAll: ({index printString. value printString. baseArray printString.} join: ', ');
		nextPutAll: ')'.
	! !

!Diff methodsFor: 'accessing' stamp: 'fbs 12/28/2011 12:34'!
value
	^ value.! !

Object subclass: #PersistentUnionFind
	instanceVariableNames: 'ranks parents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind'!

!PersistentUnionFind commentStamp: 'fbs 12/28/2011 11:07' prior: 0!
A union-find data structure maintains a _partition_ of objects in some structure. That is, it implicitly defines an equivalence relation between the nodes of some structure.

The structure provides two methods - _find_ returns the representative of some class of nodes, and _union_ merges two classes of nodes.

* ranks is an array of the ranks (sizes) of the various equivalence classes.
* parents is an array of the representative terms for each of the equivalence classes.!

Object subclass: #Ref
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind'!

!Ref commentStamp: 'fbs 12/29/2011 13:59' prior: 0!
I wrap some other object, acting something like a pointer. Thus, things may have an immutable pointer to me, while having my value be mutable: an immutable reference to a mutable thing.!

Ref subclass: #PersistentArray
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind'!

!PersistentArray commentStamp: 'fbs 12/29/2011 11:06' prior: 0!
A PersistentArray is a reference to an array-like structure that permits one fast access to the _most recent version_ of the array, but also (slower) access to _previous versions_ of the array.!

!PersistentArray class methodsFor: 'instance creation' stamp: 'fbs 12/29/2011 11:38'!
initially: anArray
	^ self new wrapping: (Arr wrapping: anArray).! !

!PersistentArray methodsFor: 'converting' stamp: 'fbs 12/29/2011 11:18'!
asArray
	^ self value asArray.! !

!PersistentArray methodsFor: 'accessing' stamp: 'fbs 12/29/2011 13:10'!
at: anInteger
	| t |
	t := value.
	^ t isDiff
		ifFalse: [t array at: anInteger]
		ifTrue: [ | t1 |
			self reroot.
			t1 := value.
			t1 isDiff
				ifFalse: [Error signal: 'Reroot returned a Diff']
				ifTrue: [t1 array at: anInteger]]! !

!PersistentArray methodsFor: 'accessing' stamp: 'fbs 12/29/2011 13:14'!
at: anInteger put: anObject
	| t |
	self reroot.
	t := value.
	^ t isDiff
		ifTrue: [Error signal: 'Reroot returns a Diff']
		ifFalse: [ | t1 |
			t1 := value.
			t1 isDiff
				ifFalse: [| old res |
					old := t1 array at: anInteger.
					t1 array at: anInteger put: anObject.
					res := PersistentArray wrapping: t1.
					self value: (Diff index: anInteger value: old in: res).
					res]
				ifTrue: [PersistentArray wrapping:
							(Diff
								index: anInteger
								value: anObject
								in: self)]].! !

!PersistentArray methodsFor: 'updating' stamp: 'fbs 12/29/2011 13:52'!
reroot
	| t t1 |
	t := value.
	t isDiff ifFalse: [^ self].
	
	t baseArray reroot.
	t1 := t baseArray value.
	t1 isDiff
		ifFalse: [ | v1 |
			v1 := t baseArray at: t index.
			t baseArray at: t index put: t value.
			self value: t1.
			t baseArray value: (Diff index: t index value: v1 in: self)]
		ifTrue: [Error signal: 'Reroot returned a Diff']! !

!Ref class methodsFor: 'instance creation' stamp: 'fbs 12/29/2011 11:13'!
wrapping: anObject
	^ self new wrapping: anObject.! !

!Ref methodsFor: 'printing' stamp: 'fbs 12/29/2011 11:20'!
printOn: aStream
	aStream nextPutAll: ('Ref({1})' format: {value printString})! !

!Ref methodsFor: 'accessing' stamp: 'fbs 12/29/2011 10:38'!
value
	^ value.! !

!Ref methodsFor: 'accessing' stamp: 'fbs 12/29/2011 11:13'!
value: anObject
	value := anObject.! !

!Ref methodsFor: 'private' stamp: 'fbs 12/29/2011 11:14'!
wrapping: anObject
	value := anObject.! !

TestCase subclass: #ArrTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind-Tests'!

!ArrTest methodsFor: 'testing - converting' stamp: 'fbs 12/28/2011 12:38'!
testAsArrayReturnsArray
	self assert: #(1 2 3) equals: (Arr wrapping: #(1 2 3)) asArray.! !

TestCase subclass: #DiffTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind-Tests'!

TestCase subclass: #PersistentArrayTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind-Tests'!

!PersistentArrayTest methodsFor: 'testing - accessing' stamp: 'fbs 12/29/2011 11:39'!
testAtPutLeavesWrappedArrayUnchanged
	| array persistentArray |
	array := #(3 2 1).
	persistentArray := PersistentArray initially: array.
	persistentArray := persistentArray at: 1 put: 4.

	self assert: #(3 2 1) equals: array.! !

!PersistentArrayTest methodsFor: 'testing - accessing' stamp: 'fbs 12/29/2011 11:39'!
testAtPutReturnsaPersistentArray
	| array persistentArray persistentArray2 |
	array := #(3 2 1).
	persistentArray := PersistentArray initially: array.
	persistentArray2 := persistentArray at: 1 put: 4.
	self assert: #(4 2 1) equals: persistentArray2 asArray.! !

!PersistentArrayTest methodsFor: 'testing - accessing' stamp: 'fbs 12/29/2011 11:39'!
testAtPutWorksOnImmediateReferences
	| array persistentArray |
	array := #(3 2 1).
	persistentArray := PersistentArray initially: array.
	persistentArray := persistentArray at: 1 put: 4.
	self assert: 4 equals: (persistentArray at: 1).! !

!PersistentArrayTest methodsFor: 'testing - accessing' stamp: 'fbs 12/29/2011 11:40'!
testAtWorksOnImmediateReferences
	| array persistentArray |
	array := #(3 2 1).
	persistentArray := PersistentArray initially: array.
	self assert: 3 equals: (persistentArray at: 1).
	self assert: 2 equals: (persistentArray at: 2).
	self assert: 1 equals: (persistentArray at: 3).! !

!PersistentArrayTest methodsFor: 'testing - accessing' stamp: 'fbs 12/29/2011 13:48'!
testPersistentArraysPersist
	| arr arr2 arr3 arr4 |
	arr := PersistentArray initially: #(1 2 3).
	arr2 := arr at: 1 put: 4.
	arr3 := arr2 at: 2 put: 5.
	arr4 := arr2 at: 3 put: 6.
	
	"A linear chain of changes."
	self assert: #(1 2 3) equals: arr asArray.
	self assert: #(4 2 3) equals: arr2 asArray.
	self assert: #(4 2 6) equals: arr4 asArray.
	
	"The alternative branch of changes."
	self assert: #(4 5 3) equals: arr3 asArray.! !

!PersistentArrayTest methodsFor: 'testing - printing' stamp: 'fbs 12/29/2011 13:58'!
testPrintOnPrintsSensiblyForNew
	| a |
	a := PersistentArray initially: #(1 2 3).
	self assert: 'Ref(Arr(#(1 2 3)))' equals: a printString.! !

!PersistentArrayTest methodsFor: 'testing - printing' stamp: 'fbs 12/29/2011 13:56'!
testPrintOnPrintsSensiblyForUpdatedArrays
	| d |
	d := Diff index: 1 value: 4 in: (PersistentArray initially: #(1 2 3)).
	self assert: 'Diff(1, 4, Ref(Arr(#(1 2 3))))' equals: d printString.! !

!PersistentArrayTest methodsFor: 'testing - updating' stamp: 'fbs 12/29/2011 13:48'!
testRerootLeavesArraySeeminglyUnchanged
	| arr arr2 arr3 |
	arr := PersistentArray initially: #(1 2 3).
	arr2 := arr at: 3 put: 4.
	arr3 := arr2 reroot.
	
	self assert: #(1 2 4) equals: arr3 asArray! !
